/*    
** housekeeper: the housekeeper game.
** Copyright (C) 2002~2007  Feynman Software

** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.

** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.

** You should have received a copy of the GNU General Public License along
** with this program; if not, write to the Free Software Foundation, Inc.,
** 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#include "map.h"

#define ID_GOTO		7980
#define ID_NEXT		7990
#define ID_PREVIOUS     8000 
#define ID_RESTART	8010
#define ID_UNDO		8020
#define ID_EXIT		8040

#define	ID_MAP_A	8050
#define	ID_MAP_B	8060
#define	ID_MAP_C	8070
#define	ID_MAP_D	8080
#define	ID_MAP_E	8090
#define	ID_MAP_F	8100
#define	ID_MAP_G	8110
#define	ID_MAP_H	8120
#define ID_ABOUT	8130
	
#define  WIDTH_LARGEWIN   600
#define  HEIGHT_LARGEWIN  400

int TestMyWinProc(HWND hWnd,int message,WPARAM wParam,LPARAM lParam);
void InitMyWinCreateInfo(PMAINWINCREATE pCreateInfo);
HMENU createmenu1(void);
HMENU createpmenugame(void);
HMENU createpmenucollection(void);
HMENU createpmenuabout(void);
BOOL InitializeApp(void);
void OnClose(HWND hWnd);
void OnDraw(HWND hWnd,HDC hDC);

static BITMAP bitmapAll[6];
static const char* lang= "en";
static const char**pmain_menu =NULL;
static const char**pgame_menu =NULL;
static const char**pabout_menu =NULL;
static const char**pcollection_menu =NULL;
static const char**pmessagebox =NULL;


static const char* messagebox_cn[] ={
    "选择游戏级别",
    "游戏级别",
    "确定",
    "取消",
    "推箱子－1.0\n\nCopyright (C) 2003 ~ 2008 北京飞漫软件技术有限公司\n\n 保留所有权利",
    "推箱子",
    "任务完成",
    "你行!"
};

static const char* messagebox_tw[] ={
    "選擇游戲級別",
    "游戲級別",
    "確定",
    "取消",
    "推箱子－1.0\n\nCopyright (C) 2003 ~ 2008 北京飛漫軟件技術有限公司\n\n 保留所有權利",
    "推箱子",
    "任務完成",
    "你行!"
};

static const char* messagebox_en[] ={
    "Choose level",          //0
    "Game Level:",           //1
    "Ok",                    //2
    "Cancel",                //3
    "House Keeper -- Version 1.0\n\nCopyright (C) 2003 ~ 2008 Beijing Feynman Software Technology Co., Ltd.\nAll rights reserved.",   //4
    "House Keeper",          //5
    "Misson Completed!",     //6
    "Good"                   //7
};

static const char* collection_menu_cn[] ={
    "排列集合",
    "排列集合 A",
    "排列集合 B",
    "排列集合 C",
    "排列集合 D",
    "排列集合 E",
    "排列集合 F",
    "排列集合 G",
    "排列集合 H"
};

static const char* collection_menu_tw[] ={
    "排列集合",
    "排列集合 A",
    "排列集合 B",
    "排列集合 C",
    "排列集合 D",
    "排列集合 E",
    "排列集合 F",
    "排列集合 G",
    "排列集合 H"
};

static const char* collection_menu_en[] ={
    "Collection",
    "Collection A",
    "Collection B",
    "Collection C",
    "Collection D",
    "Collection E",
    "Collection F",
    "Collection G",
    "Collection H"
};

static const char* about_menu_cn[] ={
    "关于",
    "关于推箱子游戏"
};

static const char* about_menu_tw[] ={
    "關於",
    "關於推箱子游戲"
};

static const char* about_menu_en[] ={
    "About",
    "About House Keeper"
};

static const char* game_menu_cn[] ={
    "游戏",
    "选择级別",
    "下一级別",
    "上一级別",
    "重新开始",
    "撤消",
    "退出"
};

static const char* game_menu_tw[] ={
    "游戲",
    "選擇級別",
    "下一級別",
    "上一級別",
    "重新開始",
    "撤消",
    "退出"
};

static const char* game_menu_en[] ={
    "Game",                      //0
    "Goto Level",                //1
    "Next Level",                //2
    "Previous Level",            //3
    "Restart Level",             //4
    "Undo",                      //5
    "Exit"                      //6
};

static const char* main_menu_cn[] ={
    "游戏",
    "排列集合",
    "关于"
};

static const char* main_menu_tw[] ={
    "游戲",
    "排列集合",
    "關於"
};

static const char* main_menu_en[] ={
    "Game",
    "Collection",
    "About"
};


static PLOGFONT cap_font = NULL;
static PLOGFONT ctrl_font = NULL;
static PLOGFONT menu_font = NULL;
static PLOGFONT utf8_font = NULL;

static void init_housekeeper(void)
{

    cap_font = g_SysLogFont[SYSLOGFONT_CAPTION];
    ctrl_font = g_SysLogFont[SYSLOGFONT_CONTROL];
    menu_font = g_SysLogFont[SYSLOGFONT_MENU];

    utf8_font = CreateLogFontByName("*-Arial-rrncnn-*-12-UTF-8");

    g_SysLogFont[SYSLOGFONT_CAPTION] = utf8_font;
    g_SysLogFont[SYSLOGFONT_CONTROL] = utf8_font;
    g_SysLogFont[SYSLOGFONT_MENU] = utf8_font;

    if(strcasecmp(lang,"zh_cn")==0){
        local_SysText = GetSysTextInUTF8("zh_CN");
    }
    else if(strcasecmp(lang,"zh_tw")==0){
        local_SysText = GetSysTextInUTF8("zh_TW");
    }
    else
       local_SysText = GetSysTextInUTF8("EN");
}

static void release_housekeeper(void)
{
    g_SysLogFont[SYSLOGFONT_CAPTION] = cap_font;
    g_SysLogFont[SYSLOGFONT_CONTROL] = ctrl_font;
    g_SysLogFont[SYSLOGFONT_MENU] = menu_font;

    DestroyLogFont(utf8_font);
}
#define IDC_LEVELINPUT 9000
DLGTEMPLATE DlgInputLevel = 
{
    WS_BORDER | WS_CAPTION,
    WS_EX_IMECOMPOSE,
    70, 80, 400, 158,
    "Choose level",
    0, 0,
    4, NULL,
    0
};

CTRLDATA CtrlInputLevel [] = 
{
    {
        "static",
        WS_VISIBLE | SS_RIGHT,
        14, 30, 150, 18,
        IDC_STATIC, 
        "Game Level:",
        0
    },
    {
        "edit",
        WS_VISIBLE | WS_BORDER | WS_TABSTOP, 
        180, 30, 100, 22, 
        IDC_LEVELINPUT, 
        NULL,
        0
    },
    {
        "button",
        WS_VISIBLE | BS_PUSHBUTTON | WS_TABSTOP, 
        80, 94, 100, 28, 
        IDOK, 
        "Ok",
        0 
    },
    {
        "button",
        WS_VISIBLE | BS_PUSHBUTTON | WS_TABSTOP,
        256, 94, 100, 28, 
        IDCANCEL, 
        "Cancel", 
        0
    }
};
  
static int 
DialogInputLevelProc(HWND hDlg, int message, WPARAM wParam, LPARAM lParam)
{
	char strText[4];
	int i;

	switch (message) {
	case MSG_INITDIALOG:
        	return 1;
        
    	case MSG_COMMAND:
		switch (wParam) {
		case IDOK:
			GetDlgItemText(hDlg,IDC_LEVELINPUT,strText,3);
			i = atoi(strText);
			EndDialog(hDlg,i);
			break;
		case IDCANCEL:
		    EndDialog (hDlg, 0);
		    break;
		}
		break;
    }
    
    return DefaultDialogProc (hDlg, message, wParam, lParam);
}
HMENU createmenu1(void)
{
    HMENU hmnu;
    MENUITEMINFO mii;

    hmnu = CreateMenu();

    memset (&mii, 0, sizeof(MENUITEMINFO));
    mii.type        = MFT_STRING;
    mii.id          = 100;
    mii.typedata    = (DWORD)pmain_menu[0];
    mii.hsubmenu    = createpmenugame();

    InsertMenuItem(hmnu, 0, TRUE, &mii);

    mii.type        = MFT_STRING;
    mii.id          = 110;
    mii.typedata    = (DWORD)pmain_menu[1];
    mii.hsubmenu    = createpmenucollection();
    InsertMenuItem(hmnu, 1, TRUE, &mii);
    
    mii.type        = MFT_STRING;
    mii.id          = 120;
    mii.typedata    = (DWORD)pmain_menu[2];
    mii.hsubmenu    = createpmenuabout();
    InsertMenuItem(hmnu, 2, TRUE, &mii);
                   
    return hmnu;
}

HMENU createpmenuabout(void)
{
    HMENU hmnu;
    MENUITEMINFO mii;
    memset (&mii, 0, sizeof(MENUITEMINFO));
    mii.type        = MFT_STRING;
    mii.id          = 0;
    mii.typedata    = (DWORD)pabout_menu[0];
    hmnu = CreatePopupMenu (&mii);
    
    memset (&mii, 0, sizeof(MENUITEMINFO));
    mii.type        = MFT_STRING ;
    mii.state       = 0;
    mii.id          = ID_ABOUT;
    mii.typedata    = (DWORD)pabout_menu[1];
    InsertMenuItem(hmnu, 0, TRUE, &mii);
    return hmnu;
}

HMENU createpmenugame(void)
{
    HMENU hmnu;

    MENUITEMINFO mii;
    memset (&mii, 0, sizeof(MENUITEMINFO));
    mii.type        = MFT_STRING;
    mii.id          = 0;
    mii.typedata    = (DWORD)pgame_menu[0];
    hmnu = CreatePopupMenu (&mii);
    
    memset (&mii, 0, sizeof(MENUITEMINFO));
    mii.type        = MFT_STRING ;
    mii.state       = 0;
    mii.id          = ID_GOTO;
    mii.typedata    = (DWORD)pgame_menu[1];
    InsertMenuItem(hmnu, 0, TRUE, &mii);

    memset (&mii, 0, sizeof(MENUITEMINFO));
    mii.type        = MFT_STRING ;
    mii.state       = 0;
    mii.id          = ID_NEXT;
    mii.typedata    = (DWORD)pgame_menu[2];
    InsertMenuItem(hmnu, 1, TRUE, &mii);

    memset (&mii, 0, sizeof(MENUITEMINFO));
    mii.type        = MFT_STRING ;
    mii.state       = 0;
    mii.id          = ID_PREVIOUS;
    mii.typedata    = (DWORD)pgame_menu[3];
    InsertMenuItem(hmnu, 2, TRUE, &mii);
    
    mii.type        = MFT_STRING ;
    mii.state       = 0;
    mii.id          = ID_RESTART;
    mii.typedata    = (DWORD)pgame_menu[4];
    InsertMenuItem(hmnu, 3, TRUE, &mii);
    
    mii.type        = MFT_STRING ;
    mii.state       = 0;
    mii.id          = ID_UNDO;
    mii.typedata    = (DWORD)pgame_menu[5];
    InsertMenuItem(hmnu, 4, TRUE, &mii);


    mii.type        = MFT_STRING ;
    mii.state       = 0;
    mii.id          = ID_EXIT;
    mii.typedata    = (DWORD)pgame_menu[6];
    InsertMenuItem(hmnu, 5, TRUE, &mii);
    
    return hmnu;
}

HMENU createpmenucollection(void)
{
    HMENU hmnu;

    MENUITEMINFO mii;
    memset (&mii, 0, sizeof(MENUITEMINFO));
    mii.type        = MFT_STRING;
    mii.id          = 0;
    mii.typedata    = (DWORD)pcollection_menu[0];
    hmnu = CreatePopupMenu (&mii);
    

    memset (&mii, 0, sizeof(MENUITEMINFO));
    mii.type        = MFT_STRING ;
    mii.state       = 0;
    mii.id          = ID_MAP_A;
    mii.typedata    = (DWORD)pcollection_menu[1];
    InsertMenuItem(hmnu, 0, TRUE, &mii);

    memset (&mii, 0, sizeof(MENUITEMINFO));
    mii.type        = MFT_STRING ;
    mii.state       = 0;
    mii.id          = ID_MAP_B;
    mii.typedata    = (DWORD)pcollection_menu[2];
    InsertMenuItem(hmnu, 1, TRUE, &mii);

    memset (&mii, 0, sizeof(MENUITEMINFO));
    mii.type        = MFT_STRING ;
    mii.state       = 0;
    mii.id          = ID_MAP_C;
    mii.typedata    = (DWORD)pcollection_menu[3];
    InsertMenuItem(hmnu, 2, TRUE, &mii);


    memset (&mii, 0, sizeof(MENUITEMINFO));
    mii.type        = MFT_STRING ;
    mii.state       = 0;
    mii.id          = ID_MAP_D;
    mii.typedata    = (DWORD)pcollection_menu[4];
    InsertMenuItem(hmnu, 3, TRUE, &mii);


    memset (&mii, 0, sizeof(MENUITEMINFO));
    mii.type        = MFT_STRING ;
    mii.state       = 0;
    mii.id          = ID_MAP_E;
    mii.typedata    = (DWORD)pcollection_menu[5];
    InsertMenuItem(hmnu, 4, TRUE, &mii);


    memset (&mii, 0, sizeof(MENUITEMINFO));
    mii.type        = MFT_STRING ;
    mii.state       = 0;
    mii.id          = ID_MAP_F;
    mii.typedata    = (DWORD)pcollection_menu[6];
    InsertMenuItem(hmnu, 5, TRUE, &mii);


    memset (&mii, 0, sizeof(MENUITEMINFO));
    mii.type        = MFT_STRING ;
    mii.state       = 0;
    mii.id          = ID_MAP_G;
    mii.typedata    = (DWORD)pcollection_menu[7];
    InsertMenuItem(hmnu, 6, TRUE, &mii);


    memset (&mii, 0, sizeof(MENUITEMINFO));
    mii.type        = MFT_STRING ;
    mii.state       = 0;
    mii.id          = ID_MAP_H;
    mii.typedata    = (DWORD)pcollection_menu[8];
    InsertMenuItem(hmnu, 7, TRUE, &mii);

    return hmnu;
}

int TestMyWinProc(HWND hWnd, int message, WPARAM wParam, LPARAM lParam)
{
	HDC hdc;
	int iRet;
	int x,y;
	ptagLevelCollection pTempColl;
	ptagLevel pTempLev;
	int iDir = 0;
	//char strTemp[50];
	switch(message){
	case MSG_CREATE:
		if(!InitializeApp()){
            return 1;
		}

        CtrlInputLevel[0].caption =pmessagebox[1];
        CtrlInputLevel[2].caption =pmessagebox[2];
        CtrlInputLevel[3].caption =pmessagebox[3];
    	DlgInputLevel.controls = CtrlInputLevel;
        DlgInputLevel.caption =pmessagebox[0];
		break;
	case MSG_COMMAND:
		switch(LOWORD(wParam)){	
		case ID_GOTO:
    			iRet = DialogBoxIndirectParam(&DlgInputLevel, hWnd, DialogInputLevelProc, 0L);
			if ((iRet > 0) && (iRet <= theMap->current->iNoOfLevels)) {
				pTempLev = theMap->current->head;
				if (pTempLev->iNo != iRet) {
					pTempLev = pTempLev->next;
					while(pTempLev != theMap->current->head) {
						if(pTempLev->iNo == iRet) 
							break;
						pTempLev = pTempLev->next;
					}
				}	
			        if ((pTempLev->iNo == iRet) && (pTempLev != theMap->currentLevel)) {
					theMap->current->current = pTempLev;
					PlayRestart();
					InvalidateRect(hWnd, NULL, TRUE);
				}		
			}
			break;
		case ID_NEXT:
			theMap->current->current = theMap->current->current->next;
			PlayRestart();
			InvalidateRect(hWnd, NULL, TRUE);
			break;
		case ID_PREVIOUS:
			theMap->current->current = theMap->current->current->prev;
			PlayRestart();
			InvalidateRect(hWnd, NULL, TRUE);
			break;
		case ID_RESTART:
			PlayRestart();
			InvalidateRect(hWnd, NULL, TRUE);
			break;
		case ID_UNDO:
			PlayUndo(hWnd); 
			break;
		case ID_EXIT:
		 	OnClose(hWnd);
			break;
		case ID_MAP_A:
			pTempColl = theMap->head;
			if (GotoCollection(pTempColl)) InvalidateRect(hWnd, NULL, TRUE);
			break;
		case ID_MAP_B:
			pTempColl = theMap->head->next;
			if (GotoCollection(pTempColl)) InvalidateRect(hWnd, NULL, TRUE);
			break;
		case ID_MAP_C:
			pTempColl = theMap->head->next->next;
			if (GotoCollection(pTempColl)) InvalidateRect(hWnd, NULL, TRUE);
			break;
		case ID_MAP_D:
			pTempColl = theMap->head->next->next->next;
			if (GotoCollection(pTempColl)) InvalidateRect(hWnd, NULL, TRUE);
			break;
		case ID_MAP_E:
			pTempColl = theMap->tail->prev->prev->prev;
			if (GotoCollection(pTempColl)) InvalidateRect(hWnd, NULL, TRUE);
			break;
		case ID_MAP_F:
			pTempColl = theMap->tail->prev->prev;
			if (GotoCollection(pTempColl)) InvalidateRect(hWnd, NULL, TRUE);
			break;
		case ID_MAP_G:
			pTempColl = theMap->tail->prev;
			if (GotoCollection(pTempColl)) InvalidateRect(hWnd, NULL, TRUE);
			break;
		case ID_MAP_H:
			pTempColl = theMap->tail;
			if (GotoCollection(pTempColl)) InvalidateRect(hWnd, NULL, TRUE);
			break;
		case ID_ABOUT:				
            MessageBox (hWnd, 
                pmessagebox[4],
                pmessagebox[5],
                MB_OK | MB_ICONINFORMATION);
			break;
		}		
		return 0;
	case MSG_LBUTTONDOWN:
		x = LOWORD(lParam);
		y = HIWORD(lParam);

		if (!ptChoosePlace(hWnd, &x, &y))
			return 0;
		//snprintf(strTemp, 50, "%d:%d", x, y);
		//MessageBox(HDC_SCREEN, strTemp, strTemp, MB_OK);
		PlayMove(hWnd, x, y);
		return 0;
	case MSG_RBUTTONDOWN:
		PlayUndo(hWnd); 
		return 0;
	case MSG_PAINT:
		hdc = BeginPaint(hWnd);
		OnDraw(hWnd,hdc);
		EndPaint(hWnd,hdc);
		return 0;

	case MSG_KEYDOWN:
		switch (wParam) {
			case SCANCODE_CURSORBLOCKUP:     
				iDir = DIR_UP;
				break;
			case SCANCODE_CURSORBLOCKLEFT:        
				iDir = DIR_LEFT;
				break;
			case SCANCODE_CURSORBLOCKRIGHT:      
				iDir = DIR_RIGHT;
				break;
			case SCANCODE_CURSORBLOCKDOWN:        
				iDir = DIR_DOWN;
				break;
			default:
				iDir = -1;
		}
		
		if ((iDir != -1) && (PlayKeyboard(hWnd, iDir) == STEP_TYPE_PUSH)){
			if (CheckMissionComplete()) {
				MessageBox(HDC_SCREEN, pmessagebox[6], pmessagebox[7], MB_OK);
				theMap->current->current = theMap->current->current->next;
				PlayRestart();
				InvalidateRect(hWnd, NULL, TRUE);
			}
			return 0;
		}
		
		if (wParam == SCANCODE_U) 
			PlayUndo(hWnd); 
		return 0;			
	case MSG_TIMER:
		return 0;			
    case MSG_CLOSE:
		 OnClose(hWnd);
	    	 return 0;
	}
	return DefaultMainWinProc(hWnd, message, wParam, lParam);
}

void OnDraw(HWND hWnd,HDC hDC) 
{
	RECT rect;
	POINT poStart1;
	int iSideLength;
	ptagLevel cur_level;
	char strText[50];
	int iRow;
	int iCol;
	int i, j;
	int iBmp;

	GetClientRect(hWnd,&rect);
	SetBkColor(hDC,RGB2Pixel(HDC_SCREEN, 102, 102, 102));
	//SetTextColor(hDC,COLOR_green);	
	SetPenColor(hDC,COLOR_black);
	poStart1.x = rect.left;
	poStart1.y = rect.top;
	
	cur_level = theMap->currentLevel;
	snprintf(strText, 50, "Collection %s, Level %d ", theMap->current->strName, cur_level->iNo);	
	TextOut(hDC, rect.left + 40, rect.bottom - 20, strText);
	rect.bottom -= 20;	
	if (rect.bottom - rect.top < 100)
		return;
	iRow = cur_level->row;
	iCol = cur_level->col;

	i = RECTH(rect)/iRow;
	j = RECTW(rect)/iCol;
	if (i <= j) 
		iSideLength = i;
	else 	
		iSideLength = j;	

	for (i = 0; i < iRow; i++) {
		for (j = 0; j < iCol; j++) {
			iBmp = cur_level->data[i * iCol + j];
			if (iBmp < B_NOTHING) {
				FillBoxWithBitmap(hDC, poStart1.x + iSideLength * j \
						, poStart1.y + iSideLength * i, iSideLength, iSideLength, &bitmapAll[iBmp]);
            }
		}
    }
}

void InitMyWinCreateInfo(PMAINWINCREATE pCreateInfo)
{
    pCreateInfo->dwStyle = WS_CAPTION | WS_BORDER | WS_SYSMENU |  WS_VISIBLE| WS_DLGFRAME;
    pCreateInfo->dwExStyle = WS_EX_NONE;
    pCreateInfo->spCaption=pmessagebox[5];
    pCreateInfo->hMenu = createmenu1();
    pCreateInfo->hCursor = GetSystemCursor(0);
    pCreateInfo->hIcon = 0;
    pCreateInfo->MainWindowProc = TestMyWinProc;
    pCreateInfo->lx = 512 -WIDTH_LARGEWIN/2;
    pCreateInfo->ty = 384 -HEIGHT_LARGEWIN/2;
    pCreateInfo->rx =pCreateInfo->lx+ WIDTH_LARGEWIN;
    pCreateInfo->by =pCreateInfo->ty+ HEIGHT_LARGEWIN;
    pCreateInfo->iBkColor = RGB2Pixel(HDC_SCREEN, 102, 102, 102); 
    pCreateInfo->dwAddData = 0;
    pCreateInfo->hHosting = HWND_DESKTOP;
}

#ifndef _LITE_VERSION
#include <minigui/dti.c>
#endif

int MiniGUIMain (int args, const char* arg[])
{
    MSG Msg;
    MAINWINCREATE CreateInfo;
    HWND hMainWnd;

#if defined(_LITE_VERSION) && !(_STAND_ALONE)
    int i;
    const char* layer = NULL;
    
    for (i = 1; i < args; i++) {
        if (strcmp (arg[i], "-layer") == 0) {
            layer = arg[i + 1];
           // break;
        }
        else if( strcmp( arg[i], "-lang") == 0)
             lang = arg[i+1];

    }
    //GetLayerInfo (layer, NULL, NULL, NULL);
    if (JoinLayer (layer, arg[0], 0, 0) == INV_LAYER_HANDLE) {
        printf ("JoinLayer: invalid layer handle.\n");
        exit (1);
    }
#endif
    if(strcasecmp(lang,"zh_cn")==0){
        printf("\nlanguage=%s",lang);
        pmain_menu =main_menu_cn;
        pgame_menu =game_menu_cn;
        pabout_menu =about_menu_cn;
        pcollection_menu =collection_menu_cn;
        pmessagebox =messagebox_cn;
        printf("\n2language=%s",lang);
    }
    else if(strcasecmp(lang,"zh_tw")==0){
        pmain_menu =main_menu_tw;
        pgame_menu =game_menu_tw;
        pabout_menu =about_menu_tw;
        pcollection_menu =collection_menu_tw;
        pmessagebox =messagebox_tw;
    }
    else if(strcasecmp(lang,"en")==0){
        pmain_menu =main_menu_en;
        pgame_menu =game_menu_en;
        pabout_menu =about_menu_en;
        pcollection_menu =collection_menu_en;
        pmessagebox =messagebox_en;
    }

    init_housekeeper();
    InitMyWinCreateInfo(&CreateInfo);

    hMainWnd = CreateMainWindow(&CreateInfo);

    if (hMainWnd == HWND_INVALID)
        return -1;

    ShowWindow(hMainWnd,SW_SHOWNORMAL);
    while( GetMessage(&Msg, hMainWnd) ) {
        TranslateMessage (&Msg);
        DispatchMessage(&Msg);
    }

    release_housekeeper();
    MainWindowThreadCleanup(hMainWnd);
    return 0;
}

BOOL InitializeApp(void)
{

	if(LoadBitmap(HDC_SCREEN, &bitmapAll[B_GOAL], "res/goal.gif"))
		return FALSE;
	if(LoadBitmap(HDC_SCREEN, &bitmapAll[B_MAN], "res/man.gif"))
		return FALSE;
	if(LoadBitmap(HDC_SCREEN, &bitmapAll[B_OBJECT], "res/object.gif"))
		return FALSE;
	if(LoadBitmap(HDC_SCREEN, &bitmapAll[B_SAVEMAN], "res/saveman.gif"))
		return FALSE;
	if(LoadBitmap(HDC_SCREEN, &bitmapAll[B_STONE], "res/stone.gif"))
		return FALSE;
	if(LoadBitmap(HDC_SCREEN, &bitmapAll[B_TREASURE], "res/treasure.gif"))
		return FALSE;
	InitMap();
	return TRUE;
}

void OnClose(HWND hWnd)
{
	DestroyMap();
	UnloadBitmap(&bitmapAll[B_GOAL]);
	UnloadBitmap(&bitmapAll[B_MAN]);
	UnloadBitmap(&bitmapAll[B_OBJECT]);
	UnloadBitmap(&bitmapAll[B_SAVEMAN]);
	UnloadBitmap(&bitmapAll[B_STONE]);
	UnloadBitmap(&bitmapAll[B_TREASURE]);
	DestroyMainWindow (hWnd);
	PostQuitMessage (hWnd);
}

void CovertCoord(HWND hWnd,int *px, int *py)
{
	RECT rect;
	int iSideLength;
	ptagLevel cur_level;
	int iRow;
	int iCol;
	int i, j;

	GetClientRect(hWnd,&rect);
	rect.bottom -= 20;	
	if (rect.bottom - rect.top < 100)
		return;
	cur_level = theMap->currentLevel;
	iRow = cur_level->row;
	iCol = cur_level->col;
	i = RECTH(rect)/iRow;
	j = RECTW(rect)/iCol;
	if (i <= j) 
		iSideLength = i;
	else 	
		iSideLength = j;	

	i = *px;
	j = *py;

	*px = rect.left + i * iSideLength;
	*py = rect.top  + j * iSideLength;
}

BOOL ptChoosePlace(HWND hWnd, int *px, int *py) 
{
	RECT rect;
	int iSideLength;
	ptagLevel cur_level;
	int iRow;
	int iCol;
	int i, j;

	GetClientRect(hWnd,&rect);
	rect.bottom -= 20;	
	if (rect.bottom - rect.top < 100)
		return FALSE;
	cur_level = theMap->currentLevel;
	iRow = cur_level->row;
	iCol = cur_level->col;
	i = RECTH(rect)/iRow;
	j = RECTW(rect)/iCol;
	if (i <= j) 
		iSideLength = i;
	else 	
		iSideLength = j;	

	i = *px;
	j = *py;

	if ( (i > rect.left) && (i < rect.left + iSideLength * iCol) &&
	     (j > rect.top ) && (j < rect.top  + iSideLength * iRow) ) {
		 *px = (i - rect.left) / iSideLength;
		 *py = (j - rect.top ) / iSideLength;
		 return TRUE;
	}
	return FALSE;
}

void DrawALittleBlock(HWND hWnd, int x, int y, int itype)	
{
	HDC hdc;
	RECT rect;
	int iSideLength;
	ptagLevel cur_level;
	int iRow;
	int iCol;
	int i, j;

	GetClientRect(hWnd,&rect);
	rect.bottom -= 20;	
	if (rect.bottom - rect.top < 100)
		return;
	cur_level = theMap->currentLevel;
	iRow = cur_level->row;
	iCol = cur_level->col;
	i = RECTH(rect)/iRow;
	j = RECTW(rect)/iCol;
	if (i <= j) 
		iSideLength = i;
	else 	
		iSideLength = j;	

	i = x;
	j = y;

	x = rect.left + i * iSideLength;
	y = rect.top  + j * iSideLength;

	hdc = GetClientDC(hWnd);
	if(itype <= B_NOTHING) {
		if(itype == B_NOTHING) {
			SetBrushColor(hdc,RGB2Pixel(HDC_SCREEN, 102, 102, 102));
			FillBox(hdc, x, y, iSideLength, iSideLength);
		}
		else {
			FillBoxWithBitmap(hdc, x, y, iSideLength, iSideLength, &bitmapAll[itype]);
        }

	}
	ReleaseDC(hdc);
}
